@@ -95,17 +95,16 @@ defmodule ModuleTest do
   end
 
   test "supports read access to module from __after_compile__/2" do
-    contents =
-      quote do
-        @after_compile __MODULE__
-        @foo 42
+    defmodule ModuleTest.NoRaise do
+      @after_compile __MODULE__
+      @foo 42
 
-        def __after_compile__(%Macro.Env{module: module}, bin) when is_binary(bin) do
-          Module.get_attribute(module, :foo)
-        end
+      def __after_compile__(%Macro.Env{module: module}, bin) when is_binary(bin) do
+        send(self(), Module.get_attribute(module, :foo))
       end
+    end
 
-    Module.create(ModuleTest.NoRaise, contents, __ENV__)
+    assert_received 42
   end
 
   test "in memory modules are tagged as so" do
